local path_mod = "__temp20191120__/"

-- Athena Cannon helper prototypes

local function make_athena_ammo_prototypes(data)
    local prototype_helper_particle_1 = {
        type = "particle",
        name = data.name.."-helper-particle-1",
        pictures = {
            {
                frame_count = 1,
                filename = "__core__/graphics/empty.png",
                size = 1
            }
        },
        life_time = 1
    }
    if data.prototype_helper_particle_1 ~= nil then
        for k, v in pairs(data.prototype_helper_particle_1) do
            prototype_helper_particle_1[k] = v
        end
    end
    local prototype_beam = {
        type = "beam",
        name = data.name.."-beam"
    }
    if data.prototype_beam ~= nil then
        for k, v in pairs(data.prototype_beam) do
            prototype_beam[k] = v
        end
    end
    local ammo_type = {
        category = data.ammo_category,
        consumption_modifier = 0,
        target_type = "position",
        action =
        {
            type = "direct",
            action_delivery =
            {
                type = "instant",
                target_effects =
                {
                    {
                        type = "create-entity",
                        entity_name = data.name.."-helper-particle-1",
                        trigger_created_entity = true,
                    }
                }
            }
        }
    }
    if data.ammo_type ~= nil then
        for k, v in pairs(data.ammo_type) do
            ammo_type[k] = v
        end
    end
    return {
        prototype_helper_particle_1,
        prototype_beam
    }, ammo_type
end

local function make_athena_fire(data)
    return
    {
        type = "fire",
        name = data.name,
        flags = {"placeable-off-grid", "not-on-map"},
        damage_per_tick = {amount = 0, type = "laser"},
        maximum_damage_multiplier = 1.0 / 0.0,
        damage_multiplier_increase_per_added_fuel = 1,
        damage_multiplier_decrease_per_tick = 0.005,
    
        uses_alternative_behavior = true,
        limit_overlapping_particles = true,
        initial_render_layer = "object",
        render_layer = "lower-object-above-shadow",
    
        spread_delay = 0,
        spread_delay_deviation = 0,
        maximum_spread_count = 0,
    
        particle_alpha = 0.6,
        particle_alpha_blend_duration = 60*5,
        --flame_alpha = 0.35,
        --flame_alpha_deviation = 0.05,
    
        emissions_per_second = 0,
    
        add_fuel_cooldown = 10,
        fade_in_duration = 1,
        fade_out_duration = 30,
    
        initial_lifetime = 180,
        lifetime_increase_by = 0,
        lifetime_increase_cooldown = 4,
        maximum_lifetime = 180,
        delay_between_initial_flames = 10,
        initial_flame_count = 1,
        burnt_patch_lifetime = 0,
        
        on_damage_tick_effect = data.on_damage_tick_effect,
        
        pictures =
        {
            {
                layers =
                {
                    {
                        filename = path_mod.."graphics/entity/athena-fire/athena-fire-1.png",
                        line_length = 8,
                        direction_count = 1,
                        width = 128,
                        height = 2048,
                        frame_count = 1,
                        shift = util.mul_shift(util.by_pixel(0, -960), data.scale),
                        tint = data.tint,
                        scale = data.scale,
                        blend_mode = "additive",
                        hr_version = {
                            filename = path_mod.."graphics/entity/athena-fire/hr-athena-fire-1.png",
                            line_length = 8,
                            direction_count = 1,
                            width = 256,
                            height = 4096,
                            frame_count = 1,
                            shift = util.mul_shift(util.by_pixel(0, -1920), 0.5 * data.scale),
                            tint = data.tint,
                            scale = 0.5 * data.scale,
                            blend_mode = "additive"
                        }
                    }
                }
            }
        }
    }
end

local athena_fire_tint = {r = 1.0, g = 0.75, b = 0.5, a = 1.0}
local athena_fire_scale = 1.0

do
    data:extend({{
        type = "ammo-category",
        name = "athena"
    }})
    local data_athena_ammo_prototypes = {
        name = "athena-cannon",
        ammo_category = "athena",
        prototype_beam = {
            flags = {"not-on-map"},
            width = 0.5,
            damage_interval = 120,
            random_target_offset = true,
            action_triggered_automatically = false,
            head =
            {
                filename = "__base__/graphics/entity/laser-turret/hr-laser-body.png",
                flags = beam_non_light_flags,
                line_length = 8,
                width = 64,
                height = 12,
                frame_count = 8,
                scale = 0.5,
                animation_speed = 0.5,
                blend_mode = "additive"
            },
            tail =
            {
                filename = "__base__/graphics/entity/laser-turret/hr-laser-end.png",
                flags = beam_non_light_flags,
                width = 110,
                height = 62,
                frame_count = 8,
                shift = util.by_pixel(11.5, 1),
                scale = 0.5,
                animation_speed = 0.5,
                blend_mode = "additive"
            },
            body =
            {
                {
                    filename = "__base__/graphics/entity/laser-turret/hr-laser-body.png",
                    flags = beam_non_light_flags,
                    line_length = 8,
                    width = 64,
                    height = 12,
                    frame_count = 8,
                    scale = 0.5,
                    animation_speed = 0.5,
                    blend_mode = "additive"
                }
            },

            light_animations =
            {
                head =
                {
                    filename = "__base__/graphics/entity/laser-turret/hr-laser-body-light.png",
                    line_length = 8,
                    width = 64,
                    height = 12,
                    frame_count = 8,
                    scale = 0.5,
                    animation_speed = 0.5,
                },
                tail =
                {
                    filename = "__base__/graphics/entity/laser-turret/hr-laser-end-light.png",
                    width = 110,
                    height = 62,
                    frame_count = 8,
                    shift = util.by_pixel(11.5, 1),
                    scale = 0.5,
                    animation_speed = 0.5,
                },
                body =
                {
                    {
                        filename = "__base__/graphics/entity/laser-turret/hr-laser-body-light.png",
                        line_length = 8,
                        width = 64,
                        height = 12,
                        frame_count = 8,
                        scale = 0.5,
                        animation_speed = 0.5,
                    }
                }
            },

            ground_light_animations =
            {
                head =
                {
                    filename = "__base__/graphics/entity/laser-turret/laser-ground-light-head.png",
                    line_length = 1,
                    width = 256,
                    height = 256,
                    repeat_count = 8,
                    scale = 0.5,
                    shift = util.by_pixel(-32, 0),
                    animation_speed = 0.5,
                    tint = {0.5, 0.05, 0.05}
                },
                tail =
                {
                    filename = "__base__/graphics/entity/laser-turret/laser-ground-light-tail.png",
                    line_length = 1,
                    width = 256,
                    height = 256,
                    repeat_count = 8,
                    scale = 0.5,
                    shift = util.by_pixel(32, 0),
                    animation_speed = 0.5,
                    tint = {0.5, 0.05, 0.05}
                },
                body =
                {
                    filename = "__base__/graphics/entity/laser-turret/laser-ground-light-body.png",
                    line_length = 1,
                    width = 64,
                    height = 256,
                    repeat_count = 8,
                    scale = 0.5,
                    animation_speed = 0.5,
                    tint = {0.5, 0.05, 0.05}
                }
            },

            working_sound =
            {
                sound =
                {
                    filename = "__base__/sound/fight/electric-beam.ogg",
                    volume = 1
                },
                max_sounds_per_type = 4
            }
        }
    }
    local athena_ammo_prototypes, athena_ammo_type = make_athena_ammo_prototypes(data_athena_ammo_prototypes)
    data:extend(athena_ammo_prototypes)
    data:extend({
        {
            type = "ammo",
            name = "athena-ammo",
            localised_name = {"item-name.athena-ammo"},
            icon = path_mod.."graphics/icons/athena-ammo.png",
            icon_size = 32,
            subgroup = "ammo",
            order = "f[athena-ammo]",
            stack_size = 1,
            ammo_type = athena_ammo_type
        },
        {
            type = "gun",
            name = "athena-cannon",
            localised_name = {"item-name.athena-cannon"},
            icon = path_mod.."graphics/icons/athena-cannon.png",
            icon_size = 32,
            flags = {"hidden"},
            subgroup = "gun",
            order = "d[athena-cannon]",
            attack_parameters =
            {
                type = "beam",
                cooldown = 180,
                movement_slow_down_factor = 0,
                range = 100,
                source_direction_count = 64,
                source_offset = {-0.15625, -0.07812},
                damage_modifier = 1,
                ammo_category = "athena"
            },
            stack_size = 5
        },
        make_athena_fire({
            name = "athena-fire",
            tint = athena_fire_tint,
            scale = athena_fire_scale,
            on_damage_tick_effect =
            {
                type = "direct",
                force = "all",
                ignore_collision_condition = true,
                filter_enabled = true,
                action_delivery =
                {
                    type = "instant",
                    target_effects =
                    {
                        {
                            type = "damage",
                            damage = {amount = 50, type = "laser"}
                        }
                    }
                }
            }
        })
    })
end
